# Developing Action Blocks:

Action blocks are easy to create but there are a few rules which need to be followed otherwise the
action block will not work.

## Format

Action blocks come in DLL libraries each of which can contain one or more action blocks. Typiccally
a single library file is used to hold multiple related functionality action blocks. For example, the
Branch library holds both the BranchConditional and BranchAlways action blocks.

The library must contain a reference to ActionBlocks.dll which will provide the necessary interface
and most of the common Action Block operations.

## Documentation

It is highly recommened to turn on the generale XML file option in the project settings which exposes
each XML comment on plugin classes and methods in an XML file. While the resulting XML file looks ugly
it can be used to document the various action blocks within the project.

## Interface

Each Action Block must implement the ActionBlocks.IActionBlock interface. This interface basically
just says that the action block must have the Active method. However, the interface is also used to
automatically detect action blocks within the library so that there is no need to provide a manifest
of the enclosed action blocks.

The Activate method always takes 3 parameters (regardless if the action block will need them or not).
The first parameter is a dictionary of objects indexed by string. This is how the action block is
passed action block specific parameters. The parameters required by the action block depends on the
action block itself and thus need to be documented somewhere for other users to be able to use the
action block. The next two parameters are the CreatureBoardAsset of the instigator and the victim.
Once again, an action block many not use all of these paramaters but the Active method must accept
them to comply with the IActionBlock interface.

## Architecture

The concept behind action blocks is to perform some small functionality and then finish. More complex
actions should be broken down into multiple action blocks so that the actions blocks remain as game
system indepednet as possible and as re-usable as possible. Obviously there will be game specific
action blocks but even those should not be named after the game system because other games systems
may also make use of the same game rule feature.

While Action Blocks run as co-routines to not block Talespire form running at the same time, they
are executed in sequential order. With Branch action blocks it is possible to jump to different
action blocks in the sequence but only one action block runs at once. Once one action block finishes,
the next will start unless that was the last action block in the sequence.

In order for this to happen, an action block needs to specifically say that it is done. This is done
using the statement: ``ActionBlocksProcessor.SetProcessorState(ActionBlocks.ProcessorState.idle);``.
If an action block finished its code but does not issue the above statement, the Ruleset Builder will
remain in the processing state and nothing will happen. This is intentional because it allows action
blocks to start and then wait for some callback before ending (such as the GetMiniSelection action
block).

Similarly this statement should not be called if the action block has not finished processing. Doing
so can cause serious issues with access and storing of data and cause action block to process with
the incorrect data.

Action Blocks should only be executed through the Activate functionality when an action block has not
been triggered by the Activate method it should not perform any processing. This means that if action
blocks patch code or use other event based code, the code should be pass trough (not do anything) when
the action block is not running.

## Initialization

Some action blocks libraries needs to setup some one time content or possibly perform patching in order
for their action blocks to work. Since action blocks are executed on demand it would normally make it
more difficult to determine if the action block has been initialized. To get around this problem the
library can contain a special action block called "Initialize" which functiona exactly the same as a
normal action block except it is automatically executed once at startup and is not placed into the list
of available action blocks afterwards. This allows a library to patch code or set up event based handlers
for the action blocks in this Initialize action block without fear that it will be executed multiple
times.

## Storage And Storage Resolution

To allows action blocks to interact with each other without having to support a large number of data
sources, action blocks use the concept of common storage. This is provided through the ActionBlocks
library using:

``ActionBlocksProcessor.Storage.Store(key,value)``
``ActionBlocksProcessor.Storage.Retrieve(key)``
``ActionBlocksProcessor.Storage.Clear(key)``
``ActionBlocksProcessor.Storage.ClearBranch(key)``

An action block and use these methods to store data in the common storage and then other action blocks
can retrieve that data assuming they know the key under which it was stored. This key could be hard
coded by the action block or in some cases the action blocks can pass the key in as a parameter.

The ClearBranch method allow you to remove all entries in the storage that begin with the given key.
This allows you to easily remove a chunk of data which is no longer needed. 

Storage persists thougout the action block sequence but does cleared at the beginning of an action
sequence. If an action block sequence needs to retain data beyond the action sequence (not typical)
then an action block needs to store it in some other way.

Besides using the ``ActionBlocksProcessor.Storage.Retrieve(key)`` method, storage values can also be
retried via string place holders by running them through the ``ActionBlocksProcessor.ReplaceStorageReferences``
method. It takes a string to be resolves and the reference to the instigator and victim (which is
passed into the Activate method of the action block. Using this method searches the provided string for
storage references (text in brace brackets) and replaces those references with actual values. When
the action block expects parameters, consider if the parameter should be run through this methods. In most
cases the the parameter should be run though this method to allow other action blocks to provide that
parameter without having to hard code it. Generally, besides slight performance, there is no danger
running a parameter through the method.

## Delay Support

Since action blocks are run as co-routines, they require at least one yield return. If appropriate
an action block can use multiple such yield returns but at least one is needed. This is typically
satisfied by adding an optional delay at the end of the action block. This not only satisfies the
yield return requirement but also allows the user to add a delay after a step without having to use
a separate Delay action block. This optional delay typically occurs just before ending execution of
the action block with code similar to:
```
yield return new WaitForSeconds(parameters.ContainsKey("Delay") ? float.Parse(parameters["Delay"].ToString()) : 0.1f);
ActionBlocksProcessor.SetProcessorState(ActionBlocks.ProcessorState.idle);
```

## Example
```
using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace LordAshes
{
    /// <summary>
    /// Holds Action Blocks and the Action Block Loader for Branch functionality
    /// </summary>
    public class Branch
    {
        /// <summary>
        /// Branches to the specified step in the current sequence
        /// 
        /// Parameters:
        /// 
        /// Step = Name of step to branch to
        /// 
        /// </summary>
        public class BranchAlways : ActionBlocks.IActionBlock
        {
            /// <summary>
            /// Method used to execute the action block as a Co-routine
            /// </summary>
            /// <param name="parameters">Dictionary of parameters</param>
            /// <param name="instigator">CreatureBoardAsset representing the instigator (selected mini)</param>
            /// <param name="victim">CreatureBoardAsset representing the victim (radial menu mini)</param>
            /// <returns>IEnumerator for co-routine processing</returns>
            public IEnumerator Activate(Dictionary<string, object> parameters, CreatureBoardAsset instigator, CreatureBoardAsset victim)
            {
                float delay = parameters.ContainsKey("Delay") ? float.Parse(parameters["Delay"].ToString()) : 0.1f;
                if (parameters.ContainsKey("Step"))
                {
                    yield return new WaitForSeconds(0.1f);
                    ActionBlocksProcessor.StepTo(Convert.ToString(parameters["Step"]));
                }
            }
        }

        /// <summary>
        /// Branches to the specified step in the current sequence
        /// 
        /// Parameters:
        /// 
        /// Condition = [Required] Exprssion evaluating to true or false
        /// True = [Optional] Name of the step to branch to if the expression is true
        /// False = [Optional] Name of the step to branch to if the expression is false
        /// 
        /// </summary>
        public class BranchOnCondition : ActionBlocks.IActionBlock
        {
            /// <summary>
            /// Method used to execute the action block as a Co-routine
            /// </summary>
            /// <param name="parameters">Dictionary of parameters</param>
            /// <param name="instigator">CreatureBoardAsset representing the instigator (selected mini)</param>
            /// <param name="victim">CreatureBoardAsset representing the victim (radial menu mini)</param>
            /// <returns>IEnumerator for co-routine processing</returns>
            public IEnumerator Activate(Dictionary<string, object> parameters, CreatureBoardAsset instigator, CreatureBoardAsset victim)
            {
                if (parameters.ContainsKey("Condition")) 
                {
                    yield return new WaitForSeconds(0.1f);
                    string formula = ActionBlocksProcessor.ReplaceStorageReferences(Convert.ToString(parameters["Condition"]),instigator,victim);
                    if (bool.Parse(ActionBlocksProcessor.Calculate(formula).ToString()))
                    {
                        Debug.Log("Ruleset Builder Plugin: Action Blocks: Branching Via True If Present");
                        if (parameters.ContainsKey("True"))
                        {
                            ActionBlocksProcessor.StepTo(Convert.ToString(parameters["True"]));
                        }
                        else
                        {
                            ActionBlocksProcessor.SetProcessorState(ActionBlocks.ProcessorState.idle);
                        }
                    }
                    else
                    {
                        Debug.Log("Ruleset Builder Plugin: Action Blocks: Branching Via False If Present");
                        if (parameters.ContainsKey("False"))
                        {
                            ActionBlocksProcessor.StepTo(Convert.ToString(parameters["False"]));
                        }
                        else
                        {
                            ActionBlocksProcessor.SetProcessorState(ActionBlocks.ProcessorState.idle);
                        }
                    }
                }
            }
        }
    }
}
```

